@@ -1,14 +1,17 @@
+/* exported PlaceInfo, RootInfo, PlacesManager */
 /*
  * Credits: this file is a copy of GNOME's 'placeDisplay.js' file from the 'Places Status Indicator' extension.
  * https://gitlab.gnome.org/GNOME/gnome-shell-extensions/-/blob/main/extensions/places-menu/placeDisplay.js
  */
+
+const {Gio, GLib, Shell} = imports.gi;
 const Me = imports.misc.extensionUtils.getCurrentExtension();
-const {Gio, GLib, Shell } = imports.gi;
 const Gettext = imports.gettext.domain(Me.metadata['gettext-domain']);
+const _ = Gettext.gettext;
+
 const Main = imports.ui.main;
 const ShellMountOperation = imports.ui.shellMountOperation;
 const Signals = imports.signals;
-const _ = Gettext.gettext;
 
 Gio._promisify(Gio.AppInfo, 'launch_default_for_uri_async');
 Gio._promisify(Gio.File.prototype, 'mount_enclosing_volume');
@@ -22,7 +25,7 @@ const Hostname1Iface = '<node> \
 </node>';
 const Hostname1 = Gio.DBusProxy.makeProxyWrapper(Hostname1Iface);
 
-var PlaceInfo = class ArcMenu_PlaceInfo {
+var PlaceInfo = class ArcMenuPlaceInfo {
     constructor(...params) {
         this._init(...params);
     }
@@ -31,7 +34,7 @@ var PlaceInfo = class ArcMenu_PlaceInfo {
         this.kind = kind;
         this.file = file;
         this.name = name || this._getFileName();
-        this.icon = icon ? new Gio.ThemedIcon({ name: icon }) : this.getIcon();
+        this.icon = icon ? new Gio.ThemedIcon({name: icon}) : this.getIcon();
     }
 
     destroy() {
@@ -50,10 +53,10 @@ var PlaceInfo = class ArcMenu_PlaceInfo {
                 return;
             }
 
-            let source = {
+            const source = {
                 get_icon: () => this.icon,
             };
-            let op = new ShellMountOperation.ShellMountOperation(source);
+            const op = new ShellMountOperation.ShellMountOperation(source);
             try {
                 await this.file.mount_enclosing_volume(0, op.mountOp, null);
 
@@ -69,7 +72,7 @@ var PlaceInfo = class ArcMenu_PlaceInfo {
     }
 
     launch(timestamp) {
-        let launchContext = global.create_app_launch_context(timestamp, -1);
+        const launchContext = global.create_app_launch_context(timestamp, -1);
         this._ensureMountAndLaunch(launchContext, true);
     }
 
@@ -80,7 +83,7 @@ var PlaceInfo = class ArcMenu_PlaceInfo {
             null,
             (file, result) => {
                 try {
-                    let info = file.query_info_finish(result);
+                    const info = file.query_info_finish(result);
                     this.icon = info.get_symbolic_icon();
                     this.emit('changed');
                 } catch (e) {
@@ -94,24 +97,24 @@ var PlaceInfo = class ArcMenu_PlaceInfo {
         // icon from the query info above
         switch (this.kind) {
         case 'network':
-            return new Gio.ThemedIcon({ name: 'folder-remote-symbolic' });
+            return new Gio.ThemedIcon({name: 'folder-remote-symbolic'});
         case 'devices':
-            return new Gio.ThemedIcon({ name: 'drive-harddisk-symbolic' });
+            return new Gio.ThemedIcon({name: 'drive-harddisk-symbolic'});
         case 'special':
         case 'bookmarks':
         default:
             if (!this.file.is_native())
-                return new Gio.ThemedIcon({ name: 'folder-remote-symbolic' });
+                return new Gio.ThemedIcon({name: 'folder-remote-symbolic'});
             else
-                return new Gio.ThemedIcon({ name: 'folder-symbolic' });
+                return new Gio.ThemedIcon({name: 'folder-symbolic'});
         }
     }
 
     _getFileName() {
-        if(this.file.get_path() === GLib.get_home_dir())
+        if (this.file.get_path() === GLib.get_home_dir())
             return _('Home');
         try {
-            let info = this.file.query_info('standard::display-name', 0, null);
+            const info = this.file.query_info('standard::display-name', 0, null);
             return info.get_display_name();
         } catch (e) {
             if (e instanceof Gio.IOErrorEnum)
@@ -119,15 +122,15 @@ var PlaceInfo = class ArcMenu_PlaceInfo {
             throw e;
         }
     }
-}
+};
 Signals.addSignalMethods(PlaceInfo.prototype);
 
-var RootInfo = class ArcMenu_RootInfo extends PlaceInfo {
+var RootInfo = class ArcMenuRootInfo extends PlaceInfo {
     _init() {
         super._init('devices', Gio.File.new_for_path('/'), _('Computer'));
 
-        let busName = 'org.freedesktop.hostname1';
-        let objPath = '/org/freedesktop/hostname1';
+        const busName = 'org.freedesktop.hostname1';
+        const objPath = '/org/freedesktop/hostname1';
         new Hostname1(Gio.DBus.system, busName, objPath, (obj, error) => {
             if (error)
                 return;
@@ -140,7 +143,7 @@ var RootInfo = class ArcMenu_RootInfo extends PlaceInfo {
     }
 
     getIcon() {
-        return new Gio.ThemedIcon({ name: 'drive-harddisk-symbolic' });
+        return new Gio.ThemedIcon({name: 'drive-harddisk-symbolic'});
     }
 
     _propertiesChanged(proxy) {
@@ -154,7 +157,7 @@ var RootInfo = class ArcMenu_RootInfo extends PlaceInfo {
 
     destroy() {
         if (this._proxy) {
-            if(this._propChangedId){
+            if (this._propChangedId) {
                 this._proxy.disconnect(this._propChangedId);
                 this._propChangedId = null;
             }
@@ -165,8 +168,7 @@ var RootInfo = class ArcMenu_RootInfo extends PlaceInfo {
     }
 };
 
-
-var PlaceDeviceInfo = class ArcMenu_PlaceDeviceInfo extends PlaceInfo {
+var PlaceDeviceInfo = class ArcMenuPlaceDeviceInfo extends PlaceInfo {
     _init(kind, mount) {
         this._mount = mount;
         super._init(kind, mount.get_root(), mount.get_name());
@@ -180,8 +182,16 @@ var PlaceDeviceInfo = class ArcMenu_PlaceDeviceInfo extends PlaceInfo {
         return this._mount.can_eject() || this._mount.can_unmount();
     }
 
+    canUnmount() {
+        return this._mount.can_unmount();
+    }
+
+    canEject() {
+        return this._mount.can_eject();
+    }
+
     eject() {
-        let unmountArgs = [
+        const unmountArgs = [
             Gio.MountUnmountFlags.NONE,
             new ShellMountOperation.ShellMountOperation(this._mount).mountOp,
             null, // Gio.Cancellable
@@ -213,12 +223,12 @@ var PlaceDeviceInfo = class ArcMenu_PlaceDeviceInfo extends PlaceInfo {
     }
 
     _reportFailure(exception) {
-        let msg = _('Ejecting drive “%s” failed:').format(this._mount.get_name());
+        const msg = _('Ejecting drive “%s” failed:').format(this._mount.get_name());
         Main.notifyError(msg, exception.message);
     }
 };
 
-var PlaceVolumeInfo = class ArcMenu_PlaceVolumeInfo extends PlaceInfo {
+var PlaceVolumeInfo = class ArcMenuPlaceVolumeInfo extends PlaceInfo {
     _init(kind, volume) {
         this._volume = volume;
         super._init(kind, volume.get_activation_root(), volume.get_name());
@@ -233,7 +243,7 @@ var PlaceVolumeInfo = class ArcMenu_PlaceVolumeInfo extends PlaceInfo {
         this._volume.mount(0, null, null, (volume, result) => {
             volume.mount_finish(result);
 
-            let mount = volume.get_mount();
+            const mount = volume.get_mount();
             this.file = mount.get_root();
             super.launch(timestamp);
         });
@@ -252,7 +262,7 @@ const DEFAULT_DIRECTORIES = [
     GLib.UserDirectory.DIRECTORY_VIDEOS,
 ];
 
-var PlacesManager = class ArcMenu_PlacesManager {
+var PlacesManager = class ArcMenuPlacesManager {
     constructor() {
         this._places = {
             special: [],
@@ -260,8 +270,8 @@ var PlacesManager = class ArcMenu_PlacesManager {
             bookmarks: [],
             network: [],
         };
-
-        this._settings = new Gio.Settings({ schema_id: BACKGROUND_SCHEMA });
+        this._connections = new Map();
+        this._settings = new Gio.Settings({schema_id: BACKGROUND_SCHEMA});
         this._showDesktopIconsChangedId = this._settings.connect(
             'changed::show-desktop-icons', this._updateSpecials.bind(this));
         this._updateSpecials();
@@ -295,6 +305,10 @@ var PlacesManager = class ArcMenu_PlacesManager {
         }
     }
 
+    setConnection(signal, callback) {
+        this._connections.set(this.connect(signal, callback), this);
+    }
+
     _connectVolumeMonitorSignals() {
         const signals = [
             'volume-added',
@@ -309,14 +323,21 @@ var PlacesManager = class ArcMenu_PlacesManager {
         ];
 
         this._volumeMonitorSignals = [];
-        let func = this._updateMounts.bind(this);
+        const func = this._updateMounts.bind(this);
         for (let i = 0; i < signals.length; i++) {
-            let id = this._volumeMonitor.connect(signals[i], func);
+            const id = this._volumeMonitor.connect(signals[i], func);
             this._volumeMonitorSignals.push(id);
         }
     }
 
     destroy() {
+        this._connections.forEach((object, id) => {
+            object.disconnect(id);
+            id = null;
+        });
+
+        this._connections = null;
+
         if (this._settings)
             this._settings.disconnect(this._showDesktopIconsChangedId);
         this._settings = null;
@@ -334,21 +355,21 @@ var PlacesManager = class ArcMenu_PlacesManager {
         this._places.special.forEach(p => p.destroy());
         this._places.special = [];
 
-        let homePath = GLib.get_home_dir();
+        const homePath = GLib.get_home_dir();
 
         this._places.special.push(new PlaceInfo(
             'special',
             Gio.File.new_for_path(homePath),
             _('Home')));
 
-        let specials = [];
-        let dirs = DEFAULT_DIRECTORIES.slice();
+        const specials = [];
+        const dirs = DEFAULT_DIRECTORIES.slice();
 
         if (this._settings.get_boolean('show-desktop-icons'))
             dirs.push(GLib.UserDirectory.DIRECTORY_DESKTOP);
 
         for (let i = 0; i < dirs.length; i++) {
-            let specialPath = GLib.get_user_special_dir(dirs[i]);
+            const specialPath = GLib.get_user_special_dir(dirs[i]);
             if (!specialPath || specialPath === homePath)
                 continue;
 
@@ -371,34 +392,25 @@ var PlacesManager = class ArcMenu_PlacesManager {
     }
 
     _updateMounts() {
-        let networkMounts = [];
-        let networkVolumes = [];
+        const networkMounts = [];
+        const networkVolumes = [];
 
         this._places.devices.forEach(p => p.destroy());
         this._places.devices = [];
         this._places.network.forEach(p => p.destroy());
         this._places.network = [];
 
-        /* Add standard places */
-        /*this._places.devices.push(new RootInfo());
-        this._places.network.push(new PlaceInfo(
-           'network',
-            Gio.File.new_for_uri('network:///'),
-            _('Network'),
-            'network-workgroup-symbolic')
-        );*/
-
         /* first go through all connected drives */
-        let drives = this._volumeMonitor.get_connected_drives();
+        const drives = this._volumeMonitor.get_connected_drives();
         for (let i = 0; i < drives.length; i++) {
-            let volumes = drives[i].get_volumes();
+            const volumes = drives[i].get_volumes();
 
             for (let j = 0; j < volumes.length; j++) {
-                let identifier = volumes[j].get_identifier('class');
+                const identifier = volumes[j].get_identifier('class');
                 if (identifier && identifier.includes('network')) {
                     networkVolumes.push(volumes[j]);
                 } else {
-                    let mount = volumes[j].get_mount();
+                    const mount = volumes[j].get_mount();
                     if (mount)
                         this._addMount('devices', mount);
                 }
@@ -406,23 +418,23 @@ var PlacesManager = class ArcMenu_PlacesManager {
         }
 
         /* add all volumes that is not associated with a drive */
-        let volumes = this._volumeMonitor.get_volumes();
+        const volumes = this._volumeMonitor.get_volumes();
         for (let i = 0; i < volumes.length; i++) {
             if (volumes[i].get_drive())
                 continue;
 
-            let identifier = volumes[i].get_identifier('class');
+            const identifier = volumes[i].get_identifier('class');
             if (identifier && identifier.includes('network')) {
                 networkVolumes.push(volumes[i]);
             } else {
-                let mount = volumes[i].get_mount();
+                const mount = volumes[i].get_mount();
                 if (mount)
                     this._addMount('devices', mount);
             }
         }
 
         /* add mounts that have no volume (/etc/mtab mounts, ftp, sftp,...) */
-        let mounts = this._volumeMonitor.get_mounts();
+        const mounts = this._volumeMonitor.get_mounts();
         for (let i = 0; i < mounts.length; i++) {
             if (mounts[i].is_shadowed())
                 continue;
@@ -430,7 +442,7 @@ var PlacesManager = class ArcMenu_PlacesManager {
             if (mounts[i].get_volume())
                 continue;
 
-            let root = mounts[i].get_default_location();
+            const root = mounts[i].get_default_location();
             if (!root.is_native()) {
                 networkMounts.push(mounts[i]);
                 continue;
@@ -439,7 +451,7 @@ var PlacesManager = class ArcMenu_PlacesManager {
         }
 
         for (let i = 0; i < networkVolumes.length; i++) {
-            let mount = networkVolumes[i].get_mount();
+            const mount = networkVolumes[i].get_mount();
             if (mount) {
                 networkMounts.push(mount);
                 continue;
@@ -456,7 +468,7 @@ var PlacesManager = class ArcMenu_PlacesManager {
     }
 
     _findBookmarksFile() {
-        let paths = [
+        const paths = [
             GLib.build_filenamev([GLib.get_user_config_dir(), 'gtk-3.0', 'bookmarks']),
             GLib.build_filenamev([GLib.get_home_dir(), '.gtk-bookmarks']),
         ];
@@ -472,19 +484,19 @@ var PlacesManager = class ArcMenu_PlacesManager {
     _reloadBookmarks() {
         this._bookmarks = [];
 
-        let content = Shell.get_file_contents_utf8_sync(this._bookmarksFile.get_path());
-        let lines = content.split('\n');
+        const content = Shell.get_file_contents_utf8_sync(this._bookmarksFile.get_path());
+        const lines = content.split('\n');
 
-        let bookmarks = [];
+        const bookmarks = [];
         for (let i = 0; i < lines.length; i++) {
-            let line = lines[i];
-            let components = line.split(' ');
-            let [bookmark] = components;
+            const line = lines[i];
+            const components = line.split(' ');
+            const [bookmark] = components;
 
             if (!bookmark)
                 continue;
 
-            let file = Gio.File.new_for_uri(bookmark);
+            const file = Gio.File.new_for_uri(bookmark);
             if (file.is_native() && !file.query_exists(null))
                 continue;
 
@@ -550,4 +562,4 @@ var PlacesManager = class ArcMenu_PlacesManager {
         return this._places[kind];
     }
 };
-Signals.addSignalMethods(PlacesManager.prototype);
\ No newline at end of file
+Signals.addSignalMethods(PlacesManager.prototype);
